@@ -1,11 +1,13 @@
 /* tprintf.c -- test file for mpfr_printf and mpfr_vprintf
 
-Copyright 2008, 2009 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+Contributed by the Arenaire and Caramel projects, INRIA.
+
+This file is part of the GNU MPFR Library.
 
 The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MPFR Library is distributed in the hope that it will be useful, but
@@ -14,25 +16,18 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#if defined HAVE_STDARG
+#if HAVE_STDARG
 #include <stdarg.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
 
-#if HAVE_INTTYPES_H
-# include <inttypes.h> /* for intmax_t */
-#else
-# if HAVE_STDINT_H
-#  include <stdint.h>
-# endif
-#endif
-
+#include "mpfr-intmax.h"
 #include "mpfr-test.h"
 #define STDOUT_FILENO 1
 
@@ -67,7 +62,7 @@ const int prec_max_printf = 5000;
 int stdout_redirect;
 
 static void
-check (char *fmt, mpfr_t x)
+check (const char *fmt, mpfr_t x)
 {
   if (mpfr_printf (fmt, x) == -1)
     {
@@ -79,7 +74,7 @@ check (char *fmt, mpfr_t x)
 }
 
 static void
-check_vprintf (char *fmt, ...)
+check_vprintf (const char *fmt, ...)
 {
   va_list ap;
 
@@ -95,6 +90,89 @@ check_vprintf (char *fmt, ...)
   va_end (ap);
 }
 
+static void
+check_vprintf_failure (const char *fmt, ...)
+{
+  va_list ap;
+
+  va_start (ap, fmt);
+  if (mpfr_vprintf (fmt, ap) != -1)
+   {
+      putchar ('\n');
+      fprintf (stderr, "Error in mpfr_vprintf(\"%s\", ...)\n", fmt);
+
+      va_end (ap);
+      exit (1);
+    }
+  putchar ('\n');
+  va_end (ap);
+}
+
+static void
+check_invalid_format (void)
+{
+  int i = 0;
+
+  /* format in disorder */
+  check_vprintf_failure ("blah %l2.1d blah", i);
+  check_vprintf_failure ("blah %2.1#d blah", i);
+
+  /* incomplete format */
+  check_vprintf_failure ("%", i);
+  check_vprintf_failure ("% (missing conversion specifier)", i);
+  check_vprintf_failure ("missing conversion specifier %h", i);
+  check_vprintf_failure ("this should fail %.l because of missing conversion specifier "
+                         "(or doubling %%)", i);
+  check_vprintf_failure ("%L", i);
+  check_vprintf_failure ("%hh. ", i);
+  check_vprintf_failure ("blah %j.");
+  check_vprintf_failure ("%ll blah");
+  check_vprintf_failure ("blah%t blah");
+  check_vprintf_failure ("%z ");
+  check_vprintf_failure ("%F (missing conversion specifier)");
+  check_vprintf_failure ("%Q (missing conversion specifier)");
+  check_vprintf_failure ("%M (missing conversion specifier)");
+  check_vprintf_failure ("%N (missing conversion specifier)");
+  check_vprintf_failure ("%Z (missing conversion specifier)");
+  check_vprintf_failure ("%R (missing conversion specifier)");
+  check_vprintf_failure ("%R");
+  check_vprintf_failure ("%P (missing conversion specifier)");
+
+  /* conversion specifier with wrong length specifier */
+  check_vprintf_failure ("%ha", i);
+  check_vprintf_failure ("%hhe", i);
+  check_vprintf_failure ("%jf", i);
+  check_vprintf_failure ("%lg", i);
+  check_vprintf_failure ("%tA", i);
+  check_vprintf_failure ("%zE", i);
+  check_vprintf_failure ("%Ld", i);
+  check_vprintf_failure ("%Qf", i);
+  check_vprintf_failure ("%MG", i);
+  check_vprintf_failure ("%Na", i);
+  check_vprintf_failure ("%ZE", i);
+  check_vprintf_failure ("%PG", i);
+  check_vprintf_failure ("%Fu", i);
+  check_vprintf_failure ("%Rx", i);
+}
+
+static void
+check_long_string (void)
+{
+  /* this test is VERY expensive both in time (~1 min on core2 @ 2.40GHz) and
+     in memory (~2.5 GB) */
+  mpfr_t x;
+
+  mpfr_init2 (x, INT_MAX);
+
+  mpfr_set_ui (x, 1, MPFR_RNDN);
+  mpfr_nextabove (x);
+
+  check_vprintf_failure ("%Rb", x);
+  check_vprintf_failure ("%RA %RA %Ra %Ra", x, x, x, x);
+
+  mpfr_clear (x);
+}
+
 static void
 check_special (void)
 {
@@ -145,8 +223,10 @@ static void
 check_mixed (void)
 {
   int ch = 'a';
+#ifndef NPRINTF_HH
   signed char sch = -1;
   unsigned char uch = 1;
+#endif
   short sh = -1;
   unsigned short ush = 1;
   int i = -1;
@@ -156,15 +236,19 @@ check_mixed (void)
   unsigned long ulo = 1;
   float f = -1.25;
   double d = -1.25;
+#if !defined(NPRINTF_T) || !defined(NPRINTF_L)
   long double ld = -1.25;
+#endif
 
-  ptrdiff_t p = 1;
+#ifndef NPRINTF_T
+  ptrdiff_t p = 1, saved_p;
+#endif
   size_t sz = 1;
 
   mpz_t mpz;
   mpq_t mpq;
   mpf_t mpf;
-  mp_rnd_t rnd = GMP_RNDN;
+  mpfr_rnd_t rnd = MPFR_RNDN;
 
   mpfr_t mpfr;
   mpfr_prec_t prec;
@@ -176,7 +260,7 @@ check_mixed (void)
   mpf_init (mpf);
   mpf_set_q (mpf, mpq);
   mpfr_init (mpfr);
-  mpfr_set_f (mpfr, mpf, GMP_RNDN);
+  mpfr_set_f (mpfr, mpf, MPFR_RNDN);
   prec = mpfr_get_prec (mpfr);
 
   check_vprintf ("a. %Ra, b. %u, c. %lx%n", mpfr, ui, ulo, &j);
@@ -188,25 +272,29 @@ check_mixed (void)
   check_vprintf ("a. %hi, b. %f, c. %#.2Rf%n", sh, d, mpfr, &i);
   check_length (4, i, 29, d);
   check_vprintf ("a. %R*A, b. %Fe, c. %i%zn", rnd, mpfr, mpf, sz, &sz);
-  check_length (5, sz, 34, zu);
+  check_length (5, (unsigned long) sz, 34, lu); /* no format specifier '%zu' in C89 */
   check_vprintf ("a. %Pu, b. %c, c. %RUG, d. %Zi%Zn", prec, ch, mpfr, mpz, &mpz);
   check_length_with_cmp (6, mpz, 24, mpz_cmp_ui (mpz, 24), Zi);
-  check_vprintf ("%% a. %#.0RNg, b. %Qx%Rn c. %p", mpfr, mpq, &mpfr, &i);
-  check_length_with_cmp (7, mpfr, 16, mpfr_cmp_ui (mpfr, 16), Rg);
+  check_vprintf ("%% a. %#.0RNg, b. %Qx%Rn c. %p",
+                 mpfr, mpq, &mpfr, (void *) &i);
+  check_length_with_cmp (7, mpfr, 15, mpfr_cmp_ui (mpfr, 15), Rg);
 
 #ifndef NPRINTF_T
+  saved_p = p;
   check_vprintf ("%% a. %RNg, b. %Qx, c. %td%tn", mpfr, mpq, p, &p);
-  check_length (8, p, 21, td);
+  if (p != 20)
+    mpfr_fprintf (stderr, "Error in test 8, got '%% a. %RNg, b. %Qx, c. %td'\n", mpfr, mpq, saved_p);
+  check_length (8, (long) p, 20, ld); /* no format specifier '%td' in C89 */
 #endif
 
 #ifndef NPRINTF_L
   check_vprintf ("a. %RA, b. %Lf, c. %QX%zn", mpfr, ld, mpq, &sz);
-  check_length (9, sz, 30, zu);
+  check_length (9, (unsigned long) sz, 30, lu); /* no format specifier '%zu' in C89 */
 #endif
 
 #ifndef NPRINTF_HH
   check_vprintf ("a. %hhi, b. %Ra, c. %hhu%hhn", sch, mpfr, uch, &uch);
-  check_length (10, uch, 22, hhu);
+  check_length (10, (unsigned int) uch, 22, u); /* no format specifier '%hhu' in C89 */
 #endif
 
 #if defined(HAVE_LONG_LONG) && !defined(NPRINTF_LL)
@@ -216,8 +304,8 @@ check_mixed (void)
 
     check_vprintf ("a. %Re, b. %llx%Qn", mpfr, ullo, &mpq);
     check_length_with_cmp (11, mpq, 16, mpq_cmp_ui (mpq, 16, 1), Qu);
-    check_vprintf ("a. %lli, b. %Rf%Fn", llo, mpfr, &mpf);
-    check_length_with_cmp (12, mpf, 12, mpf_cmp_ui (mpf, 12), Fg);
+    check_vprintf ("a. %lli, b. %Rf%lln", llo, mpfr, &ullo);
+    check_length (12, ullo, 19, llu);
   }
 #endif
 
@@ -226,10 +314,10 @@ check_mixed (void)
     intmax_t im = -1;
     uintmax_t uim = 1;
 
-    check_vprintf ("a. %*RA, b. %ji%Qn", 10, mpfr, im, &mpq);
-    check_length_with_cmp (31, mpq, 20, mpq_cmp_ui (mpq, 20, 1), Qu);
-    check_vprintf ("a. %.*Re, b. %jx%Fn", 10, mpfr, uim, &mpf);
-    check_length_with_cmp (32, mpf, 25, mpf_cmp_ui (mpf, 25), Fg);
+    check_vprintf ("a. %*RA, b. %ji%Fn", 10, mpfr, im, &mpf);
+    check_length_with_cmp (31, mpf, 20, mpf_cmp_ui (mpf, 20), Fg);
+    check_vprintf ("a. %.*Re, b. %jx%jn", 10, mpfr, uim, &im);
+    check_length (32, (long) im, 25, li); /* no format specifier "%ji" in C89 */
   }
 #endif
 
@@ -244,7 +332,7 @@ check_random (int nb_tests)
 {
   int i;
   mpfr_t x;
-  mp_rnd_t rnd;
+  mpfr_rnd_t rnd;
   char flag[] =
     {
       '-',
@@ -262,7 +350,7 @@ check_random (int nb_tests)
       'f',
       'g'
     };
-  mp_exp_t old_emin, old_emax;
+  mpfr_exp_t old_emin, old_emax;
 
   old_emin = mpfr_get_emin ();
   old_emax = mpfr_get_emax ();
@@ -279,15 +367,18 @@ check_random (int nb_tests)
       char *ptr = fmt;
 
       tests_default_random (x, 256, MPFR_EMIN_MIN, MPFR_EMAX_MAX);
-      rnd = (mp_rnd_t) RND_RAND ();
+      rnd = (mpfr_rnd_t) RND_RAND ();
 
       spec = (int) (randlimb () % 5);
       jmax = (spec == 3 || spec == 4) ? 6 : 5; /* ' flag only with %f or %g */
       /* advantage small precision */
       prec = (randlimb () % 2) ? 10 : prec_max_printf;
       prec = (int) (randlimb () % prec);
-      if (spec == 3 && mpfr_get_exp (x) > prec_max_printf)
-        /*  change style 'f' to style 'e' when number x is large */
+      if (spec == 3
+          && (mpfr_get_exp (x) > prec_max_printf
+              || mpfr_get_exp (x) < -prec_max_printf))
+        /*  change style 'f' to style 'e' when number x is very large or very
+            small*/
         --spec;
 
       *ptr++ = '%';
@@ -372,8 +463,14 @@ main (int argc, char *argv[])
       N = atoi (argv[1]);
     }
 
+  check_invalid_format ();
   check_special ();
   check_mixed ();
+
+  /* expensive tests */
+  if (getenv ("MPFR_CHECK_LARGEMEM") != NULL)
+    check_long_string();
+
   check_random (N);
 
   if (stdout_redirect)
@@ -402,7 +499,7 @@ int
 main (void)
 {
   /* We have nothing to test. */
-  return 0;
+  return 77;
 }
 
 #endif  /* HAVE_STDARG */
