@@ -1,24 +1,24 @@
 /* tsum -- test file for the list summation function
 
-Copyright 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+Contributed by the Arenaire and Caramel projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -36,7 +36,7 @@ check_is_sorted (unsigned long n, mpfr_srcptr *perm)
 }
 
 static int
-sum_tab (mpfr_ptr ret, mpfr_t *tab, unsigned long n, mp_rnd_t rnd)
+sum_tab (mpfr_ptr ret, mpfr_t *tab, unsigned long n, mpfr_rnd_t rnd)
 {
   mpfr_ptr *tabtmp;
   unsigned long i;
@@ -54,24 +54,43 @@ sum_tab (mpfr_ptr ret, mpfr_t *tab, unsigned long n, mp_rnd_t rnd)
 }
 
 
-static mp_prec_t
-get_prec_max (mpfr_t *tab, unsigned long n, mp_prec_t f)
+static mpfr_prec_t
+get_prec_max (mpfr_t *tab, unsigned long n, mpfr_prec_t f)
 {
-  mp_prec_t res;
-  mp_exp_t min, max;
+  mpfr_prec_t res;
+  mpfr_exp_t min, max;
   unsigned long i;
 
-  for (i = 0; MPFR_IS_ZERO (tab[i]); i++)
-    MPFR_ASSERTD (i < n);
+  i = 0;
+  while (MPFR_IS_ZERO (tab[i]))
+    {
+      i++;
+      if (i == n)
+        return MPFR_PREC_MIN;  /* all values are 0 */
+    }
+
+  if (! mpfr_check (tab[i]))
+    {
+      printf ("tab[%lu] is not valid.\n", i);
+      exit (1);
+    }
+  MPFR_ASSERTN (MPFR_IS_FP (tab[i]));
   min = max = MPFR_GET_EXP(tab[i]);
   for (i++; i < n; i++)
     {
-      if (!MPFR_IS_ZERO (tab[i])) {
-        if (MPFR_GET_EXP(tab[i]) > max)
-          max = MPFR_GET_EXP(tab[i]);
-        if (MPFR_GET_EXP(tab[i]) < min)
-          min = MPFR_GET_EXP(tab[i]);
-      }
+      if (! mpfr_check (tab[i]))
+        {
+          printf ("tab[%lu] is not valid.\n", i);
+          exit (1);
+        }
+      MPFR_ASSERTN (MPFR_IS_FP (tab[i]));
+      if (! MPFR_IS_ZERO (tab[i]))
+        {
+          if (MPFR_GET_EXP(tab[i]) > max)
+            max = MPFR_GET_EXP(tab[i]);
+          if (MPFR_GET_EXP(tab[i]) < min)
+            min = MPFR_GET_EXP(tab[i]);
+        }
     }
   res = max - min;
   res += f;
@@ -81,17 +100,17 @@ get_prec_max (mpfr_t *tab, unsigned long n, mp_prec_t f)
 
 
 static void
-algo_exact (mpfr_t somme, mpfr_t *tab, unsigned long n, mp_prec_t f)
+algo_exact (mpfr_t somme, mpfr_t *tab, unsigned long n, mpfr_prec_t f)
 {
   unsigned long i;
-  mp_prec_t prec_max;
+  mpfr_prec_t prec_max;
 
   prec_max = get_prec_max(tab, n, f);
   mpfr_set_prec (somme, prec_max);
-  mpfr_set_ui (somme, 0, GMP_RNDN);
+  mpfr_set_ui (somme, 0, MPFR_RNDN);
   for (i = 0; i < n; i++)
     {
-      if (mpfr_add(somme, somme, tab[i], GMP_RNDN))
+      if (mpfr_add(somme, somme, tab[i], MPFR_RNDN))
         {
           printf ("FIXME: algo_exact is buggy.\n");
           exit (1);
@@ -101,7 +120,7 @@ algo_exact (mpfr_t somme, mpfr_t *tab, unsigned long n, mp_prec_t f)
 
 /* Test the sorting function */
 static void
-test_sort (mp_prec_t f, unsigned long n)
+test_sort (mpfr_prec_t f, unsigned long n)
 {
   mpfr_t *tab;
   mpfr_ptr *tabtmp;
@@ -109,15 +128,15 @@ test_sort (mp_prec_t f, unsigned long n)
   unsigned long i;
 
   /* Init stuff */
-  tab = (mpfr_t *) malloc (n * sizeof(mpfr_t));
+  tab = (mpfr_t *) (*__gmp_allocate_func) (n * sizeof (mpfr_t));
   for (i = 0; i < n; i++)
     mpfr_init2 (tab[i], f);
-  tabtmp = (mpfr_ptr *) malloc (n * sizeof(mpfr_ptr));
-  perm = (mpfr_srcptr *) malloc (n * sizeof(mpfr_srcptr));
+  tabtmp = (mpfr_ptr *) (*__gmp_allocate_func) (n * sizeof(mpfr_ptr));
+  perm = (mpfr_srcptr *) (*__gmp_allocate_func) (n * sizeof(mpfr_srcptr));
 
   for (i = 0; i < n; i++)
     {
-      mpfr_random (tab[i]);
+      mpfr_urandomb (tab[i], RANDS);
       tabtmp[i] = tab[i];
     }
 
@@ -134,13 +153,13 @@ test_sort (mp_prec_t f, unsigned long n)
   /* Clear stuff */
   for (i = 0; i < n; i++)
     mpfr_clear (tab[i]);
-  free (tabtmp);
-  free (perm);
-  free (tab);
+  (*__gmp_free_func) (tab, n * sizeof (mpfr_t));
+  (*__gmp_free_func) (tabtmp, n * sizeof(mpfr_ptr));
+  (*__gmp_free_func) (perm, n * sizeof(mpfr_srcptr));
 }
 
 static void
-test_sum (mp_prec_t f, unsigned long n)
+test_sum (mpfr_prec_t f, unsigned long n)
 {
   mpfr_t sum, real_sum, real_non_rounded;
   mpfr_t *tab;
@@ -148,19 +167,19 @@ test_sum (mp_prec_t f, unsigned long n)
   int rnd_mode;
 
   /* Init */
-  tab = (mpfr_t *) malloc (n * sizeof(mpfr_t));
+  tab = (mpfr_t *) (*__gmp_allocate_func) (n * sizeof(mpfr_t));
   for (i = 0; i < n; i++)
     mpfr_init2 (tab[i], f);
-  mpfr_inits2 (f, sum, real_sum, real_non_rounded, (void *) 0);
+  mpfr_inits2 (f, sum, real_sum, real_non_rounded, (mpfr_ptr) 0);
 
   /* First Uniform */
   for (i = 0; i < n; i++)
-    mpfr_random (tab[i]);
+    mpfr_urandomb (tab[i], RANDS);
   algo_exact (real_non_rounded, tab, n, f);
-  for (rnd_mode = 0; rnd_mode < GMP_RND_MAX; rnd_mode++)
+  for (rnd_mode = 0; rnd_mode < MPFR_RND_MAX; rnd_mode++)
     {
-      sum_tab (sum, tab, n, (mp_rnd_t) rnd_mode);
-      mpfr_set (real_sum, real_non_rounded, (mp_rnd_t) rnd_mode);
+      sum_tab (sum, tab, n, (mpfr_rnd_t) rnd_mode);
+      mpfr_set (real_sum, real_non_rounded, (mpfr_rnd_t) rnd_mode);
       if (mpfr_cmp (real_sum, sum) != 0)
         {
           printf ("mpfr_sum incorrect.\n");
@@ -173,14 +192,15 @@ test_sum (mp_prec_t f, unsigned long n)
   /* Then non uniform */
   for (i = 0; i < n; i++)
     {
-      mpfr_random (tab[i]);
-      mpfr_set_exp (tab[i], randlimb () %1000);
+      mpfr_urandomb (tab[i], RANDS);
+      if (! mpfr_zero_p (tab[i]))
+        mpfr_set_exp (tab[i], randlimb () % 1000);
     }
   algo_exact (real_non_rounded, tab, n, f);
-  for (rnd_mode = 0; rnd_mode < GMP_RND_MAX; rnd_mode++)
+  for (rnd_mode = 0; rnd_mode < MPFR_RND_MAX; rnd_mode++)
     {
-      sum_tab (sum, tab, n, (mp_rnd_t) rnd_mode);
-      mpfr_set (real_sum, real_non_rounded, (mp_rnd_t) rnd_mode);
+      sum_tab (sum, tab, n, (mpfr_rnd_t) rnd_mode);
+      mpfr_set (real_sum, real_non_rounded, (mpfr_rnd_t) rnd_mode);
       if (mpfr_cmp (real_sum, sum) != 0)
         {
           printf ("mpfr_sum incorrect.\n");
@@ -193,8 +213,8 @@ test_sum (mp_prec_t f, unsigned long n)
   /* Clear stuff */
   for (i = 0; i < n; i++)
     mpfr_clear (tab[i]);
-  mpfr_clears (sum, real_sum, real_non_rounded, (void *) 0);
-  free (tab);
+  mpfr_clears (sum, real_sum, real_non_rounded, (mpfr_ptr) 0);
+  (*__gmp_free_func) (tab, n * sizeof(mpfr_t));
 }
 
 static
@@ -204,29 +224,29 @@ void check_special (void)
   mpfr_ptr tabp[3];
   int i;
 
-  mpfr_inits (tab[0], tab[1], tab[2], r, (void *) 0);
+  mpfr_inits (tab[0], tab[1], tab[2], r, (mpfr_ptr) 0);
   tabp[0] = tab[0];
   tabp[1] = tab[1];
   tabp[2] = tab[2];
 
-  i = mpfr_sum (r, tabp, 0, GMP_RNDN);
+  i = mpfr_sum (r, tabp, 0, MPFR_RNDN);
   if (!MPFR_IS_ZERO (r) || !MPFR_IS_POS (r) || i != 0)
     {
       printf ("Special case n==0 failed!\n");
       exit (1);
     }
 
-  mpfr_set_ui (tab[0], 42, GMP_RNDN);
-  i = mpfr_sum (r, tabp, 1, GMP_RNDN);
+  mpfr_set_ui (tab[0], 42, MPFR_RNDN);
+  i = mpfr_sum (r, tabp, 1, MPFR_RNDN);
   if (mpfr_cmp_ui (r, 42) || i != 0)
     {
       printf ("Special case n==1 failed!\n");
       exit (1);
     }
 
-  mpfr_set_ui (tab[1], 17, GMP_RNDN);
+  mpfr_set_ui (tab[1], 17, MPFR_RNDN);
   MPFR_SET_NAN (tab[2]);
-  i = mpfr_sum (r, tabp, 3, GMP_RNDN);
+  i = mpfr_sum (r, tabp, 3, MPFR_RNDN);
   if (!MPFR_IS_NAN (r) || i != 0)
     {
       printf ("Special case NAN failed!\n");
@@ -235,7 +255,7 @@ void check_special (void)
 
   MPFR_SET_INF (tab[2]);
   MPFR_SET_POS (tab[2]);
-  i = mpfr_sum (r, tabp, 3, GMP_RNDN);
+  i = mpfr_sum (r, tabp, 3, MPFR_RNDN);
   if (!MPFR_IS_INF (r) || !MPFR_IS_POS (r) || i != 0)
     {
       printf ("Special case +INF failed!\n");
@@ -244,7 +264,7 @@ void check_special (void)
 
   MPFR_SET_INF (tab[2]);
   MPFR_SET_NEG (tab[2]);
-  i = mpfr_sum (r, tabp, 3, GMP_RNDN);
+  i = mpfr_sum (r, tabp, 3, MPFR_RNDN);
   if (!MPFR_IS_INF (r) || !MPFR_IS_NEG (r) || i != 0)
     {
       printf ("Special case -INF failed!\n");
@@ -252,7 +272,7 @@ void check_special (void)
     }
 
   MPFR_SET_ZERO (tab[1]);
-  i = mpfr_sum (r, tabp, 2, GMP_RNDN);
+  i = mpfr_sum (r, tabp, 2, MPFR_RNDN);
   if (mpfr_cmp_ui (r, 42) || i != 0)
     {
       printf ("Special case 42+0 failed!\n");
@@ -260,7 +280,7 @@ void check_special (void)
     }
 
   MPFR_SET_NAN (tab[0]);
-  i = mpfr_sum (r, tabp, 3, GMP_RNDN);
+  i = mpfr_sum (r, tabp, 3, MPFR_RNDN);
   if (!MPFR_IS_NAN (r) || i != 0)
     {
       printf ("Special case NAN+0+-INF failed!\n");
@@ -268,23 +288,23 @@ void check_special (void)
     }
 
   mpfr_set_inf (tab[0], 1);
-  mpfr_set_ui  (tab[1], 59, GMP_RNDN);
+  mpfr_set_ui  (tab[1], 59, MPFR_RNDN);
   mpfr_set_inf (tab[2], -1);
-  i = mpfr_sum (r, tabp, 3, GMP_RNDN);
+  i = mpfr_sum (r, tabp, 3, MPFR_RNDN);
   if (!MPFR_IS_NAN (r) || i != 0)
     {
       printf ("Special case +INF + 59 +-INF failed!\n");
       exit (1);
     }
 
-  mpfr_clears (tab[0], tab[1], tab[2], r, (void *) 0);
+  mpfr_clears (tab[0], tab[1], tab[2], r, (mpfr_ptr) 0);
 }
 
 
 int
 main (void)
 {
-  mp_prec_t p;
+  mpfr_prec_t p;
   unsigned long n;
 
   tests_start_mpfr ();
